@@ -8,7 +8,9 @@ import (
 	"bytes"
 	"context"
 	"crypto"
+	"crypto/ecdh"
 	"crypto/elliptic"
+	"crypto/rand"
 	"crypto/x509"
 	"encoding/pem"
 	"errors"
@@ -22,26 +24,33 @@ import (
 	"strings"
 	"testing"
 	"time"
-
-	"golang.org/x/crypto/curve25519"
 )
 
 func testClientHello(t *testing.T, serverConfig *Config, m handshakeMessage) {
 	testClientHelloFailure(t, serverConfig, m, "")
 }
 
+// testFatal is a hack to prevent the compiler from complaining that there is a
+// call to t.Fatal from a non-test goroutine
+func testFatal(t *testing.T, err error) {
+	t.Helper()
+	t.Fatal(err)
+}
+
 func testClientHelloFailure(t *testing.T, serverConfig *Config, m handshakeMessage, expectedSubStr string) {
 	c, s := localPipe(t)
 	go func() {
-		cli := Client(c, testConfig, nil)
+		cli := Client(c, testConfig)
 		if ch, ok := m.(*clientHelloMsg); ok {
 			cli.vers = ch.vers
 		}
-		cli.writeRecord(recordTypeHandshake, m.marshal())
+		if _, err := cli.writeHandshakeRecord(m, nil); err != nil {
+			testFatal(t, err)
+		}
 		c.Close()
 	}()
 	ctx := context.Background()
-	conn := Server(s, serverConfig, nil)
+	conn := Server(s, serverConfig)
 	ch, err := conn.readClientHello(ctx)
 	hs := serverHandshakeState{
 		c:           conn,
@@ -192,9 +201,11 @@ func TestRenegotiationExtension(t *testing.T) {
 	c, s := localPipe(t)
 
 	go func() {
-		cli := Client(c, testConfig, nil)
+		cli := Client(c, testConfig)
 		cli.vers = clientHello.vers
-		cli.writeRecord(recordTypeHandshake, clientHello.marshal())
+		if _, err := cli.writeHandshakeRecord(clientHello, nil); err != nil {
+			testFatal(t, err)
+		}
 
 		buf := make([]byte, 1024)
 		n, err := c.Read(buf)
@@ -206,7 +217,7 @@ func TestRenegotiationExtension(t *testing.T) {
 		bufChan <- buf[:n]
 	}()
 
-	Server(s, testConfig, nil).Handshake()
+	Server(s, testConfig).Handshake()
 	buf := <-bufChan
 
 	if len(buf) < 5+4 {
@@ -251,10 +262,12 @@ func TestTLS12OnlyCipherSuites(t *testing.T) {
 	c, s := localPipe(t)
 	replyChan := make(chan any)
 	go func() {
-		cli := Client(c, testConfig, nil)
+		cli := Client(c, testConfig)
 		cli.vers = clientHello.vers
-		cli.writeRecord(recordTypeHandshake, clientHello.marshal())
-		reply, err := cli.readHandshake()
+		if _, err := cli.writeHandshakeRecord(clientHello, nil); err != nil {
+			testFatal(t, err)
+		}
+		reply, err := cli.readHandshake(nil)
 		c.Close()
 		if err != nil {
 			replyChan <- err
@@ -264,7 +277,7 @@ func TestTLS12OnlyCipherSuites(t *testing.T) {
 	}()
 	config := testConfig.Clone()
 	config.CipherSuites = clientHello.cipherSuites
-	Server(s, config, nil).Handshake()
+	Server(s, config).Handshake()
 	s.Close()
 	reply := <-replyChan
 	if err, ok := reply.(error); ok {
@@ -309,10 +322,12 @@ func TestTLSPointFormats(t *testing.T) {
 			c, s := localPipe(t)
 			replyChan := make(chan any)
 			go func() {
-				cli := Client(c, testConfig, nil)
+				cli := Client(c, testConfig)
 				cli.vers = clientHello.vers
-				cli.writeRecord(recordTypeHandshake, clientHello.marshal())
-				reply, err := cli.readHandshake()
+				if _, err := cli.writeHandshakeRecord(clientHello, nil); err != nil {
+					testFatal(t, err)
+				}
+				reply, err := cli.readHandshake(nil)
 				c.Close()
 				if err != nil {
 					replyChan <- err
@@ -322,7 +337,7 @@ func TestTLSPointFormats(t *testing.T) {
 			}()
 			config := testConfig.Clone()
 			config.CipherSuites = clientHello.cipherSuites
-			Server(s, config, nil).Handshake()
+			Server(s, config).Handshake()
 			s.Close()
 			reply := <-replyChan
 			if err, ok := reply.(error); ok {
@@ -348,11 +363,11 @@ func TestTLSPointFormats(t *testing.T) {
 func TestAlertForwarding(t *testing.T) {
 	c, s := localPipe(t)
 	go func() {
-		Client(c, testConfig, nil).sendAlert(alertUnknownCA)
+		Client(c, testConfig).sendAlert(alertUnknownCA)
 		c.Close()
 	}()
 
-	err := Server(s, testConfig, nil).Handshake()
+	err := Server(s, testConfig).Handshake()
 	s.Close()
 	var opErr *net.OpError
 	if !errors.As(err, &opErr) || opErr.Err != error(alertUnknownCA) {
@@ -364,7 +379,7 @@ func TestClose(t *testing.T) {
 	c, s := localPipe(t)
 	go c.Close()
 
-	err := Server(s, testConfig, nil).Handshake()
+	err := Server(s, testConfig).Handshake()
 	s.Close()
 	if err != io.EOF {
 		t.Errorf("Got error: %s; expected: %s", err, io.EOF)
@@ -424,78 +439,6 @@ func TestCipherSuitePreference(t *testing.T) {
 	}
 }
 
-func TestCipherSuiteTLS13(t *testing.T) {
-	run := func(t *testing.T, clientSuites, serverSuites []uint16, expected uint16) {
-		clientConfig := &Config{
-			CipherSuites:       clientSuites,
-			InsecureSkipVerify: true,
-		}
-		serverConfig := &Config{
-			CipherSuites: serverSuites,
-			Certificates: testConfig.Certificates,
-		}
-		state, _, err := testHandshake(t, clientConfig, serverConfig)
-		if err != nil {
-			t.Fatalf("handshake failed: %s", err)
-		}
-		if state.CipherSuite != expected {
-			// By default the server should use the client's preference.
-			t.Fatalf("Expected %v to be used, got %v", expected, state.CipherSuite)
-		}
-	}
-
-	t.Run("client only supports ChaCha", func(t *testing.T) {
-		run(
-			t,
-			[]uint16{TLS_CHACHA20_POLY1305_SHA256},
-			[]uint16{TLS_AES_128_GCM_SHA256, TLS_CHACHA20_POLY1305_SHA256},
-			TLS_CHACHA20_POLY1305_SHA256,
-		)
-	})
-	t.Run("server only supports ChaCha", func(t *testing.T) {
-		run(
-			t,
-			[]uint16{TLS_AES_128_GCM_SHA256, TLS_CHACHA20_POLY1305_SHA256},
-			[]uint16{TLS_CHACHA20_POLY1305_SHA256},
-			TLS_CHACHA20_POLY1305_SHA256,
-		)
-	})
-	t.Run("client only supports AES_128", func(t *testing.T) {
-		run(
-			t,
-			[]uint16{TLS_AES_128_GCM_SHA256},
-			[]uint16{TLS_AES_128_GCM_SHA256, TLS_CHACHA20_POLY1305_SHA256},
-			TLS_AES_128_GCM_SHA256,
-		)
-	})
-	t.Run("server only supports AES_128", func(t *testing.T) {
-		run(
-			t,
-			[]uint16{TLS_AES_128_GCM_SHA256, TLS_CHACHA20_POLY1305_SHA256},
-			[]uint16{TLS_AES_128_GCM_SHA256},
-			TLS_AES_128_GCM_SHA256,
-		)
-	})
-}
-
-func TestCipherSuiteConfigServerTLS13(t *testing.T) {
-	serverConfig := &Config{
-		CipherSuites: []uint16{TLS_CHACHA20_POLY1305_SHA256},
-		Certificates: testConfig.Certificates,
-	}
-	clientConfig := &Config{
-		CipherSuites:       []uint16{TLS_AES_128_GCM_SHA256, TLS_CHACHA20_POLY1305_SHA256},
-		InsecureSkipVerify: true,
-	}
-	state, _, err := testHandshake(t, clientConfig, serverConfig)
-	if err != nil {
-		t.Fatalf("handshake failed: %s", err)
-	}
-	if state.CipherSuite != TLS_CHACHA20_POLY1305_SHA256 {
-		t.Fatalf("Server's preference was not used, got %x", state.CipherSuite)
-	}
-}
-
 func TestSCTHandshake(t *testing.T) {
 	t.Run("TLSv12", func(t *testing.T) { testSCTHandshake(t, VersionTLS12) })
 	t.Run("TLSv13", func(t *testing.T) { testSCTHandshake(t, VersionTLS13) })
@@ -720,7 +663,7 @@ func (test *serverTest) run(t *testing.T, write bool) {
 	if config == nil {
 		config = testConfig
 	}
-	server := Server(serverConn, config, nil)
+	server := Server(serverConn, config)
 	connStateChan := make(chan ConnectionState, 1)
 	go func() {
 		_, err := server.Write([]byte("hello, world\n"))
@@ -985,47 +928,6 @@ func TestHandshakeServerALPN(t *testing.T) {
 	runServerTestTLS13(t, test)
 }
 
-func TestHandshakeServerEnforceALPN(t *testing.T) {
-	run := func(t *testing.T, serverProtos, clientProtos []string, expectedSuccess bool) {
-		clientConn, serverConn := localPipe(t)
-		serverConfig := testConfig.Clone()
-		serverConfig.NextProtos = serverProtos
-		client := Client(clientConn, serverConfig, nil)
-
-		cErrChan := make(chan error)
-		go func() {
-			cErrChan <- client.Handshake()
-		}()
-
-		config := testConfig.Clone()
-		config.NextProtos = clientProtos
-		extraConf := &ExtraConfig{EnforceNextProtoSelection: true}
-
-		server := Server(serverConn, config, extraConf)
-		sErr := server.Handshake()
-		cErr := <-cErrChan
-		if expectedSuccess {
-			if sErr != nil || cErr != nil {
-				t.Fatalf("Expected ALPN negotiation to succeed. Client error: %v, server error: %v", cErr, sErr)
-			}
-		} else {
-			if sErr == nil || sErr.Error() != "tls: client requested unsupported application protocols ([proto1 proto2])" {
-				t.Fatalf("Expected APLN negotiation to fail, got %v", sErr)
-			}
-			if cErr == nil || !strings.Contains(cErr.Error(), "no application protocol") {
-				t.Fatalf("Expect 'no_application_protocol' error, got %v", cErr)
-			}
-		}
-	}
-
-	t.Run("ALPN failure", func(t *testing.T) {
-		run(t, []string{"proto1", "proto2"}, []string{"proto3"}, false)
-	})
-	t.Run("ALPN negotiation success", func(t *testing.T) {
-		run(t, []string{"proto1", "proto2"}, []string{"proto1"}, true)
-	})
-}
-
 func TestHandshakeServerALPNNoMatch(t *testing.T) {
 	config := testConfig.Clone()
 	config.NextProtos = []string{"proto3"}
@@ -1356,10 +1258,10 @@ func benchmarkHandshakeServer(b *testing.B, version uint16, cipherSuite uint16,
 		config := testConfig.Clone()
 		config.MaxVersion = version
 		config.CurvePreferences = []CurveID{curve}
-		client := Client(clientConn, config, nil)
+		client := Client(clientConn, config)
 		client.Handshake()
 	}()
-	server := Server(serverConn, config, nil)
+	server := Server(serverConn, config)
 	if err := server.Handshake(); err != nil {
 		b.Fatalf("handshake failed: %v", err)
 	}
@@ -1391,7 +1293,7 @@ func benchmarkHandshakeServer(b *testing.B, version uint16, cipherSuite uint16,
 	b.ResetTimer()
 	for i := 0; i < b.N; i++ {
 		feeder <- struct{}{}
-		server := Server(serverConn, config, nil)
+		server := Server(serverConn, config)
 		if err := server.Handshake(); err != nil {
 			b.Fatalf("handshake failed: %v", err)
 		}
@@ -1537,12 +1439,14 @@ func TestSNIGivenOnFailure(t *testing.T) {
 
 	c, s := localPipe(t)
 	go func() {
-		cli := Client(c, testConfig, nil)
+		cli := Client(c, testConfig)
 		cli.vers = clientHello.vers
-		cli.writeRecord(recordTypeHandshake, clientHello.marshal())
+		if _, err := cli.writeHandshakeRecord(clientHello, nil); err != nil {
+			testFatal(t, err)
+		}
 		c.Close()
 	}()
-	conn := Server(s, serverConfig, nil)
+	conn := Server(s, serverConfig)
 	ctx := context.Background()
 	ch, err := conn.readClientHello(ctx)
 	hs := serverHandshakeState{
@@ -1562,7 +1466,6 @@ func TestSNIGivenOnFailure(t *testing.T) {
 		t.Error("No error reported from server")
 	}
 
-	hs.c.updateConnectionState()
 	cs := hs.c.ConnectionState()
 	if cs.HandshakeComplete {
 		t.Error("Handshake registered as complete")
@@ -1675,10 +1578,10 @@ func TestGetConfigForClient(t *testing.T) {
 
 		go func() {
 			defer s.Close()
-			done <- Server(s, serverConfig, nil).Handshake()
+			done <- Server(s, serverConfig).Handshake()
 		}()
 
-		clientErr := Client(c, clientConfig, nil).Handshake()
+		clientErr := Client(c, clientConfig).Handshake()
 		c.Close()
 
 		serverErr := <-done
@@ -1702,58 +1605,9 @@ func TestGetConfigForClient(t *testing.T) {
 	}
 }
 
-func TestAdditionalExtensionsReceivedByServer(t *testing.T) {
-	c, s := net.Pipe()
-	done := make(chan bool)
-
-	config := testConfig.Clone()
-	config.MinVersion = VersionTLS13
-	config.MaxVersion = VersionTLS13
-	cExtraConf := &ExtraConfig{}
-	cExtraConf.GetExtensions = func(_ uint8) []Extension {
-		return []Extension{
-			{Type: 0x1337, Data: []byte("foobar")},
-		}
-	}
-	go func() {
-		Client(s, config, cExtraConf).Handshake()
-		s.Close()
-		done <- true
-	}()
-
-	var receivedExtensions bool
-	sExtraConf := &ExtraConfig{}
-	sExtraConf.ReceivedExtensions = func(handshakeMessageType uint8, exts []Extension) {
-		receivedExtensions = true
-		if handshakeMessageType != typeClientHello {
-			t.Errorf("expected handshake message type to be %d, but got %d", typeClientHello, handshakeMessageType)
-		}
-		// TODO(#84): parse signature_algorithms_cert
-		if len(exts) == 2 && exts[0].Type == 50 {
-			exts = exts[1:]
-		}
-		if len(exts) != 1 {
-			t.Errorf("expected to received 1 extension, got %d", len(exts))
-		}
-		if exts[0].Type != 0x1337 {
-			t.Errorf("expected extension type 0x1337, got %#x", exts[0].Type)
-		}
-		if string(exts[0].Data) != "foobar" {
-			t.Errorf("expection extension data to be foobar, got %s", exts[0].Data)
-		}
-	}
-	err := Server(c, config, sExtraConf).Handshake()
-	if err != nil {
-		t.Errorf("expected client to complete handshake, got %s", err)
-	}
-	if !receivedExtensions {
-		t.Errorf("expected client to receive extensions")
-	}
-}
-
 func TestCloseServerConnectionOnIdleClient(t *testing.T) {
 	clientConn, serverConn := localPipe(t)
-	server := Server(serverConn, testConfig.Clone(), nil)
+	server := Server(serverConn, testConfig.Clone())
 	go func() {
 		clientConn.Write([]byte{'0'})
 		server.Close()
@@ -1812,13 +1666,13 @@ T+E0J8wlH24pgwQHzy7Ko2qLwn1b5PW8ecrlvP1g
 	}
 
 	clientConn, serverConn := localPipe(t)
-	client := Client(clientConn, testConfig, nil)
+	client := Client(clientConn, testConfig)
 	done := make(chan struct{})
 	go func() {
 		config := testConfig.Clone()
 		config.Certificates = []Certificate{cert}
 		config.MinVersion = VersionTLS13
-		server := Server(serverConn, config, nil)
+		server := Server(serverConn, config)
 		err := server.Handshake()
 		expectError(t, err, "key size too small")
 		close(done)
@@ -2065,6 +1919,7 @@ func TestAESCipherReorderingTLS13(t *testing.T) {
 	for _, tc := range tests {
 		t.Run(tc.name, func(t *testing.T) {
 			hasAESGCMHardwareSupport = tc.serverHasAESGCM
+			pk, _ := ecdh.X25519().GenerateKey(rand.Reader)
 			hs := &serverHandshakeStateTLS13{
 				c: &Conn{
 					config: &config{},
@@ -2074,7 +1929,7 @@ func TestAESCipherReorderingTLS13(t *testing.T) {
 					cipherSuites:       tc.clientCiphers,
 					supportedVersions:  []uint16{VersionTLS13},
 					compressionMethods: []uint8{compressionNone},
-					keyShares:          []keyShare{{group: X25519, data: curve25519.Basepoint}},
+					keyShares:          []keyShare{{group: X25519, data: pk.PublicKey().Bytes()}},
 				},
 			}
 
@@ -2103,7 +1958,7 @@ func TestServerHandshakeContextCancellation(t *testing.T) {
 		<-unblockClient
 		_ = c.Close()
 	}()
-	conn := Server(s, testConfig, nil)
+	conn := Server(s, testConfig)
 	// Initiates server side handshake, which will block until a client hello is read
 	// unless the cancellation works.
 	err := conn.HandshakeContext(ctx)
@@ -2151,7 +2006,7 @@ func TestHandshakeContextHierarchy(t *testing.T) {
 				PrivateKey:  testRSAPrivateKey,
 			}, nil
 		}
-		cli := Client(c, clientConfig, nil)
+		cli := Client(c, clientConfig)
 		err := cli.HandshakeContext(ctx)
 		if err != nil {
 			clientErr <- err
@@ -2176,7 +2031,7 @@ func TestHandshakeContextHierarchy(t *testing.T) {
 			PrivateKey:  testRSAPrivateKey,
 		}, nil
 	}
-	conn := Server(s, serverConfig, nil)
+	conn := Server(s, serverConfig)
 	err := conn.HandshakeContext(ctx)
 	if err != nil {
 		t.Errorf("Unexpected server handshake error: %v", err)
